import {difficulties, Difficulty} from "./Difficulty";
import {LocalStorage, Storage} from "./Storage";


/**
 * A score obtained by clearing a field.
 */
export type Score = {time: number, deaths: number};


/**
 * The highest scores.
 */
export class HighScores {
    private static readonly scoresToStore: number = 10;
    private readonly storage: Storage;


    constructor(storage: Storage = new LocalStorage("/tools/minesweeper//high-scores")) {
        this.storage = storage;
    }


    clear(): void {
        this.storage.clear();
    }

    addScore(difficulty: Difficulty, score: Score): void {
        const scores = this.storage.getArray(difficulty.name, []);
        scores.push(score);
        scores.sort((a, b) => a.time - b.time);
        this.storage.setArray(difficulty.name, scores.slice(0, HighScores.scoresToStore));
    }

    getScores(difficulty: Difficulty): Score[] {
        return this.storage.getArray(difficulty.name, []);
    }

    generateHtmlReport(): string {
        let report = "";
        for (let i = 0; i < difficulties.length; i++) {
            const difficulty = difficulties[i];
            const highScores = this.getScores(difficulty);

            report += "" +
                `<h3>${difficulty.name}</h3>
                <table>
                    <tr>
                        <th>Time (seconds)</th>
                        <th>Deaths</th>
                    </tr>`;
            for (let j = 0; j < highScores.length; j++) {
                const score = highScores[j];
                report += "" +
                    `<tr>
                        <td>${score.time / 1000}</td>
                        <td>${score.deaths}</td>
                    </tr>`;
            }
            report += "</table>";
        }
        return report;
    }
}
